"""Test invalid selectors."""
from .. import util


class TestInvalid(util.TestCase):
    """Test invalid selectors."""

    def test_user_invalid(self):
        """Test user invalid (matches nothing)."""

        markup = """
        <form id="form">
          <input id="1" type="text">
        </form>
        """

        self.assert_selector(
            markup,
            "input:user-invalid",
            [],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "input:not(:user-invalid)",
            ["1"],
            flags=util.HTML
        )



"""Test host selectors."""
from .. import util


class TestHost(util.TestCase):
    """Test host selectors."""

    MARKUP = """<h1>header</h1><div><p>some text</p></div>"""

    def test_host(self):
        """Test host (not supported)."""

        self.assert_selector(
            self.MARKUP,
            ":host",
            [],
            flags=util.HTML
        )

    def test_host_func(self):
        """Test host function (not supported)."""

        self.assert_selector(
            self.MARKUP,
            ":host(h1)",
            [],
            flags=util.HTML
        )



"""Test at-rule cases."""
from .. import util


class TestAtRule(util.TestCase):
    """Test at-rules."""

    def test_at_rule(self):
        """Test at-rule (not supported)."""

        self.assert_raises('@page :left', NotImplementedError)



"""Test any link selectors."""
from .. import util


class TestAnyLink(util.TestCase):
    """Test any link selectors."""

    MARKUP = """
    <div id="div">
    <p id="0">Some text <span id="1" class="foo:bar:foobar"> in a paragraph</span>.
    <a id="2" class="bar" href="http://google.com">Link</a>
    <a id="3">Placeholder text.</a>
    </p>
    </div>
    """

    def test_anylink(self):
        """Test any link (all links are unvisited)."""

        self.assert_selector(
            self.MARKUP,
            ":any-link",
            ["2"],
            flags=util.HTML
        )

    def test_anylink_xml(self):
        """Test that any link will not match in XML (all links are unvisited)."""

        self.assert_selector(
            self.MARKUP,
            "a:any-link",
            [],
            flags=util.XML
        )

    def test_anylink_xhtml(self):
        """Test that any link will not match in XHTML (all links are unvisited)."""

        self.assert_selector(
            self.wrap_xhtml(self.MARKUP),
            "a:any-link",
            ['2'],
            flags=util.XHTML
        )

    def test_not_anylink(self):
        """Test the inverse of any link (all links are unvisited)."""

        self.assert_selector(
            self.MARKUP,
            ":not(a:any-link)",
            ["div", "0", "1", "2", "3"],
            flags=util.XML
        )



"""Test optional selectors."""
from .. import util


class TestOptional(util.TestCase):
    """Test optional selectors."""

    MARKUP = """
    <form>
    <input id="1" type="name" required>
    <input id="2" type="checkbox" required>
    <input id="3" type="email">
    <textarea id="4" name="name" cols="30" rows="10"></textarea>
    <select id="5" name="nm">
        <!-- options -->
    </select>
    </form>
    """

    def test_optional(self):
        """Test optional."""

        self.assert_selector(
            self.MARKUP,
            ":optional",
            ['3', '4', '5'],
            flags=util.HTML
        )

    def test_specific_optional(self):
        """Test specific optional."""

        self.assert_selector(
            self.MARKUP,
            "input:optional",
            ['3'],
            flags=util.HTML
        )



"""Test type selectors."""
from .. import util
from soupsieve import SelectorSyntaxError


class TestType(util.TestCase):
    """Test type selectors."""

    MARKUP = """
    <Tag id="1">
    <tag id="2"></tag>
    <TAG id="3"></TAG>
    </Tag>
    """

    def test_basic_type(self):
        """Test type."""

        self.assert_selector(
            """
            <div>
            <p>Some text <span id="1"> in a paragraph</span>.</p>
            <a id="2" href="http://google.com">Link</a>
            </div>
            """,
            "span",
            ["1"],
            flags=util.HTML
        )

    def test_type_html(self):
        """Test type for HTML."""

        self.assert_selector(
            self.MARKUP,
            "tag",
            ["1", "2", "3"],
            flags=util.HTML
        )

        self.assert_selector(
            self.MARKUP,
            "Tag",
            ["1", "2", "3"],
            flags=util.HTML
        )

        self.assert_selector(
            self.MARKUP,
            "TAG",
            ["1", "2", "3"],
            flags=util.HTML
        )

    def test_type_xhtml(self):
        """Test type for XHTML."""

        self.assert_selector(
            self.wrap_xhtml(self.MARKUP),
            "tag",
            ["2"],
            flags=util.XHTML
        )

        self.assert_selector(
            self.wrap_xhtml(self.MARKUP),
            "Tag",
            ["1"],
            flags=util.XHTML
        )

        self.assert_selector(
            self.wrap_xhtml(self.MARKUP),
            "TAG",
            ["3"],
            flags=util.XHTML
        )

    def test_type_xml(self):
        """Test type for XML."""

        self.assert_selector(
            self.MARKUP,
            "tag",
            ["2"],
            flags=util.XML
        )

        self.assert_selector(
            self.MARKUP,
            "Tag",
            ["1"],
            flags=util.XML
        )

        self.assert_selector(
            self.MARKUP,
            "TAG",
            ["3"],
            flags=util.XML
        )

    def test_invalid_syntax(self):
        """Test invalid syntax."""

        self.assert_raises('div?', SelectorSyntaxError)
        self.assert_raises('-', SelectorSyntaxError)



"""Test future selectors."""
from .. import util


class TestFuture(util.TestCase):
    """Test future selectors."""

    MARKUP = """
    <body>
    <div id="div">
    <p id="0">Some text <span id="1" class="foo:bar:foobar"> in a paragraph</span>.
    <a id="2" class="bar" href="http://google.com">Link</a>
    <a id="3">Placeholder text.</a>
    </p>
    </div>
    </body>
    """

    def test_future(self):
        """Test future (should match nothing)."""

        self.assert_selector(
            self.MARKUP,
            "p:future",
            [],
            flags=util.HTML
        )

    def test_not_future(self):
        """Test not future."""

        self.assert_selector(
            self.MARKUP,
            "p:not(:future)",
            ["0"],
            flags=util.HTML
        )



"""Test placeholder shown selectors."""
from .. import util


class TestPlaceholderShown(util.TestCase):
    """Test placeholder shown selectors."""

    def test_placeholder_shown(self):
        """Test placeholder shown."""

        markup = """
        <!-- These have a placeholder. -->
        <input id="0" placeholder="This is some text">
        <textarea id="1" placeholder="This is some text"></textarea>

        <!-- These do not have a placeholder. -->
        <input id="2" placeholder="">
        <input id="3">

        <!-- All types that should register has having a placeholder. -->
        <input id="4" type="email" placeholder="This is some text">
        <input id="5" type="number" placeholder="This is some text">
        <input id="6" type="password" placeholder="This is some text">
        <input id="7" type="search" placeholder="This is some text">
        <input id="8" type="tel" placeholder="This is some text">
        <input id="9" type="text" placeholder="This is some text">
        <input id="10" type="url" placeholder="This is some text">
        <input id="11" type="" placeholder="This is some text">
        <input id="12" type placeholder="This is some text">

        <!-- Types that should not register has having a placeholder. -->
        <input id="13" type="button" placeholder="This is some text">
        <input id="14" type="checkbox" placeholder="This is some text">
        <input id="15" type="color" placeholder="This is some text">
        <input id="16" type="date" placeholder="This is some text">
        <input id="17" type="datetime-local" placeholder="This is some text">
        <input id="18" type="file" placeholder="This is some text">
        <input id="19" type="hidden" placeholder="This is some text">
        <input id="20" type="image" placeholder="This is some text">
        <input id="21" type="month" placeholder="This is some text">
        <input id="22" type="radio" placeholder="This is some text">
        <input id="23" type="range" placeholder="This is some text">
        <input id="24" type="reset" placeholder="This is some text">
        <input id="25" type="submit" placeholder="This is some text">
        <input id="26" type="time" placeholder="This is some text">
        <input id="27" type="week" placeholder="This is some text">

        <!-- Value will not override this instance as value is empty. -->
        <input id="28" type placeholder="This is some text" value="">

        <!-- Value will override this input -->
        <input id="29" type placeholder="This is some text" value="Actual value">

        <!-- Text area content overrides the placehold-->
        <textarea id="30" placeholder="This is some text">Value</textarea>
        <textarea id="31" placeholder="This is some text">


        </textarea>

        <!-- Text area is still considered empty with a single new line (does not include carriage return). -->
        <textarea id="32" placeholder="This is some text">
        </textarea>
        """

        self.assert_selector(
            markup,
            ":placeholder-shown",
            ['0', '1', '4', '5', '6', '7', '8', '9', '10', '11', '12', '28', '32'],
            flags=util.HTML
        )
